#!/usr/bin/env python
#-*- coding: utf-8 -*-

import os
import re
import sys
import service_config
from bases.log import *
from redis_utils import RedisUtils

redis_utils = RedisUtils()
hadoop_home = '/opt/hadoop-3.1.0'
dfs_path_prefix = '/production/service-jaguar'
def get_job_id_from_name(name):
    cmd = "%s/bin/hadoop job -list 2>/dev/null | grep %s | awk '{print $1}'" % (hadoop_home, name)
    logger().info('get job id cmd : %s ', cmd )
    res = os.popen(cmd).read()
    return res

def get_job_progress_from_name(name):
    cmd = "%s/bin/hadoop job -list 2>/dev/null | grep %s | awk '{print $1}' | xargs %s/bin/hadoop job -status 2>/dev/null | grep complet" % (
        hadoop_home,
        name,
        hadoop_home
    )
    res = os.popen(cmd).read()

    result = []
    for line in res.split('\n'):
        if "map" in line or 'reduce' in line:
            segs = line.split('completion:')
            pro_map = round(float(segs[1].strip()), 2)
            result.append(pro_map)

    logger().info('hadoop[%s] progress[%s]', name, str(result))
    return result

def hrm(dfs_path):
    cmd = "%s/bin/hadoop fs -rm -r -f %s" % (
        hadoop_home,
        dfs_path_prefix + dfs_path
    )
    print "hrm cmd is : " + cmd
    #return os.system(cmd)==0

def exist(dfs_path):
    cmd = "%s/bin/hadoop fs -test -e %s" % (
        hadoop_home,
        dfs_path
    )
    print cmd
    return os.system(cmd) == 0

def hdu(dfs_path):
    cmd = "%s/bin/hadoop fs -du -s %s | awk '{print $1}'" % (
        hadoop_home,
        dfs_path
    )
    print cmd
    return os.popen(cmd).read().strip()
    
def hput(local_path, dfs_path):
    cmd = "%s/bin/hadoop fs -put -f %s %s" % (hadoop_home, local_path, dfs_path)
    print 'hadoop cmd : ' + cmd
    res = os.system(cmd)==0
    print res
    return res


def hmkdir(dfs_path, recursive=True):
    cmd_fmt = "%s/bin/hadoop fs -mkdir "
    if recursive:
        cmd_fmt += '-p'
    cmd_fmt += ' %s'
    cmd = cmd_fmt % (hadoop_home, dfs_path)
    logger().info('hadoop cmd: ' + cmd)
    res = os.system(cmd) == 0
    return res


def hcat(dfs_path_file):
    cmd = "%s/bin/hadoop fs -cat %s" % (hadoop_home, dfs_path_file)
    res = os.popen(cmd).read().strip()
    return res

def getTrackPaths(taskId):
    task_path = redis_utils.get_task_path(taskId)
    logger().info('taskId : %s get task_path : %s ', taskId, str(task_path))
    track_path_list = []
    cmd = "%s/bin/hadoop fs -ls %s/track | awk '{print $8}' | cat -A" % (hadoop_home, task_path)
    logger().info('taskId : %s , get TrackPath cmd : %s ', taskId, cmd)
    for line in os.popen(cmd):
        line = line.strip()[:-1]
        if re.match("track[0-9]+", line.split(os.sep)[-1])!=None:
            track_path_list.append(line)
    return track_path_list


def main():
    print get_job_id_from_name('R_200705574_1-1')
    #print exist(sys.argv[1])
    #print get_job_progress_from_name(sys.argv[1])
    #print exist(sys.argv[0])

if __name__ == '__main__':
    main()



#!/usr/bin/env python
#-*-coding: utf-8 -*-

import os
import sys
import urllib2
from pyhdfs import HdfsClient
try:
    import jaguar_v2
    jaguar_v2.sys_append_paths()
except:
    pass
import json
from bases.log import logger
from bases.service_config import GET_CONF

client = HdfsClient(
            hosts=GET_CONF('hadoop', 'hadoop_client_hosts'),
            user_name=GET_CONF('hadoop', 'hadoop_client_user_name'),
            max_tries=int(GET_CONF('hadoop', 'hadoop_client_max_tries'))
        )
apps_url = 'http://hm-001:8088/ws/v1/cluster/apps'
def exists(hdfs_path):
    return client.exists(hdfs_path)

def delete(hdfs_path):
    res = False
    if client.exists(hdfs_path):
        res = client.delete(hdfs_path, recursive = True)
    else:
        return True
    return res

def download(hdfs_path, local_path):
    file_name = hdfs_path.split('/')[-1]
    local_path = '%s/%s'%(local_path, file_name)
    if client.exists(hdfs_path):
        try:
            client.copy_to_local(hdfs_path, local_path)
        except Exception, e:
            import traceback
            error_msg = traceback.format_exc()
            logger().error('download faill error:%s'%(error_msg))
            return False  
        return True
    else:
        return False

def download_for_auto(hdfs_path, local_path):
    file_name = hdfs_path.split('/')[-1] + '_test'
    local_path = '%s/%s'%(local_path, file_name)
    if client.exists(hdfs_path):
        try:
            client.copy_to_local(hdfs_path, local_path)
        except Exception, e:
            import traceback
            error_msg = traceback.format_exc()
            logger().error('download faill error:%s'%(error_msg))
            return False  
        return True
    else:
        return False

def download_sign(hdfs_path, local_path, sign):
    file_name = hdfs_path.split('/')[-1] + '_%s' % sign
    local_path = '%s/%s'%(local_path, file_name)
    if client.exists(hdfs_path):
        try:
            client.copy_to_local(hdfs_path, local_path)
        except Exception, e:
            import traceback
            error_msg = traceback.format_exc()
            logger().error('download faill error:%s'%(error_msg))
            return False  
        return True
    else:
        return False

def download_dir(hdfs_path, local_path):
    if hdfs_path[-1] == os.sep:
        hdfs_path = hdfs_path[:-1]
    if local_path[-1] == os.sep:
        local_path = local_path + os.sep
    local_path = local_path + hdfs_path.split(os.sep)[-1]
    hadoop_home = '/opt/hadoop-3.1.0/bin'
    cmd = 'hadoop fs -get %s %s'%(hdfs_path, local_path)
    res = os.system(cmd) == 0
    return res


def upload(local_path, hdfs_path):
    file_name = local_path.split('/')[-1]
    hdfs_path = '%s/%s'%(hdfs_path, file_name)
    logger().info('local_path:%s, hdfs_path:%s'%(local_path, hdfs_path))
    res = False
    if delete(hdfs_path):
        try:
            client.copy_from_local(local_path, hdfs_path)
        except Exception, e:
            import traceback
            error_msg = traceback.format_exc()
            logger().error('upload faill error:%s'%(error_msg))
            return False
        res = True
    return res

def listdir(hdfs_path):
    return client.listdir(hdfs_path)

def upload_dir(local_path, hdfs_path):
    hadoop_home = '/opt/hadoop-3.1.0/bin'
    cmd = '%s/hadoop fs -put -f %s %s'%(hadoop_home, local_path, hdfs_path)
    res = os.system(cmd) == 0
    return res
def get_job_id(job_name):
    def filter_app(x):
        target_name = x['name']
        return target_name == job_name
    #apps_url = 'http://hm-001:8088/ws/v1/cluster/apps'
    response = urllib2.urlopen(apps_url)
    print apps_url
    data = response.read()
    #print data
    data = json.loads(data)
    apps = data['apps']['app']
    target_apps = filter(filter_app, apps)
    if not target_apps:
        return None
    target_apps = sorted(target_apps, key=lambda x:x['startedTime'], reverse=True)
    target_app = target_apps[0]
    job_id = 'job' + target_app['id'][11:]
    return job_id

if __name__ == '__main__':
    #print 'ds'
    print get_job_id('')
    #print exists('/production/jaguar/200000143/auto/output/26_20180623055523928.tx')    
    #print upload_dir('./base','/tmp')
    #print delete('/tmp/t.txt')
    #client.copy_from_local('/tmp/t.txt', '/tmp/t.txt')
    #print download('/tmp/t.txt','./')
    #print listdir('/production/jaguar/jaguar/version1/auto/input')
    #print download_dir('/tmp/output/protocol','./')




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import redis_pool
import json
import sys
import os
import service_config
from singleton import singleton

#project_name = service_config.GET_CONF("service", "project_name")
project_name = 'jaguar'

@singleton
class RedisUtils():
    def __init__(self):
        self.r = redis_pool.Pool().get('ir_server')
    
    # task queue, 0: start task 1: stop task
    def queue_push(self, data, which):
        if which==0:
            return self.r.lpush(service_config.GET_CONF("redis", project_name + "_cmd_start_queue"), data)
        elif which==1:
            return self.r.lpush(service_config.GET_CONF("redis", project_name + "_cmd_stop_queue"), data)

    def queue_len(self, which):
        if which==0:
            return self.r.llen(service_config.GET_CONF("redis", project_name + "_cmd_start_queue"))
        elif which==1:
            return self.r.llen(service_config.GET_CONF("redis", project_name + "_cmd_stop_queue"))

    def queue_pop(self, which):
        if which==0:
            return self.r.rpop(service_config.GET_CONF("redis", project_name + "_cmd_start_queue"))
        elif which==1:
            return self.r.rpop(service_config.GET_CONF("redis", project_name + "_cmd_stop_queue"))

    # start task pid
    def set_start_pid(self, seq, start_pid):
        return self.r.hset(service_config.GET_CONF("redis", project_name + "_start_pid"), seq, start_pid)

    def get_start_pid(self, seq):
        return self.r.hget(service_config.GET_CONF("redis", project_name + "_start_pid"), seq)

    def rem_start_pid(self, seq):
        return self.r.hdel(service_config.GET_CONF("redis", project_name + "_start_pid"), seq)

    def set_cur_name(self, seq, cur_name):
        return self.r.hset(service_config.GET_CONF("redis", project_name + "_cur_name"), seq, cur_name)

    def get_cur_name(self, seq):
        return self.r.hget(service_config.GET_CONF("redis", project_name + "_cur_name"), seq)

    def rem_cur_name(self, seq):
        return self.r.hdel(service_config.GET_CONF("redis", project_name + "_cur_name"), seq)

    # start task state
    def set_state(self, seq, state):                                                                                              
        return self.r.hset(service_config.GET_CONF("redis", project_name + "_state"), seq, state)

    def get_state(self, seq):
        return self.r.hget(service_config.GET_CONF("redis", project_name + "_state"), seq)

    def rem_state(self, seq):
        return self.r.hdel(service_config.GET_CONF("redis", project_name + "_state"), seq)

    # running start task
    def add_running(self, seq):
        return self.r.sadd(service_config.GET_CONF("redis", project_name + "_running"), seq)

    def rem_running(self, seq):
        return self.r.srem(service_config.GET_CONF("redis", project_name + "_running"), seq)

    def scard_running(self):
        return self.r.scard(service_config.GET_CONF("redis", project_name + "_running"))

    def smembers_running(self):
        return self.r.smembers(service_config.GET_CONF("redis", project_name + "_running"))

    # running stop task
    def add_running_stop(self, seq):
        return self.r.sadd(service_config.GET_CONF("redis", project_name + "_running_stop"), seq)

    def rem_running_stop(self, seq):
        return self.r.srem(service_config.GET_CONF("redis", project_name + "_running_stop"), seq)

    def scard_running_stop(self):
        return self.r.scard(service_config.GET_CONF("redis", project_name + "_running_stop"))

    def smembers_running_stop(self):
        return self.r.smembers(service_config.GET_CONF("redis", project_name + "_running_stop"))

    def clear(self):
        self.r.delete(service_config.GET_CONF("redis", project_name + "_start_pid"))
        self.r.delete(service_config.GET_CONF("redis", project_name + "_state"))
        self.r.delete(service_config.GET_CONF("redis", project_name + "_running"))
        self.r.delete(service_config.GET_CONF("redis", project_name + "_running_stop"))
        self.r.delete(service_config.GET_CONF("redis", project_name + "_cur_name"))
        return True


    #process pid
    def set_process_pid(self, process_name, pid):
        return self.r.hset(service_config.GET_CONF("redis", project_name + "_process_pid"), process_name, pid)
    
    def set_type(self, project_id, task_type):
        return self.r.hset("jaguar_type", project_id, task_type)
    
    def get_type(self, project_id):
        return self.r.hget("jaguar_type", project_id)

    def get_process_pid(self):
        return self.r.hgetall(service_config.GET_CONF("redis", project_name + "_process_pid"))

    def set_auto_status(self, project_id, track_id, statu_data):
        return self.r.hset(project_id + "_auto", track_id, statu_data)
    
    def get_auto_status(self, project_id, track_id):
        ret = self.r.hget(project_id + "_auto", track_id) 
        if not ret:
            print >> sys.stderr, 'no res, return 0'
            return 0
        return ret
    
    def get_all_auto_status(self, project_id):
        return self.r.hgetall(project_id + "_auto")
    
    def rem_auto_status(self, project_id):
        return self.r.delete(project_id + "_auto")

    def set_fusion_status(self, project_id, track_id, statu_data):
        return self.r.hset(project_id + "_fusion", track_id, statu_data)
    
    def get_fusion_status(self, project_id, fusion_id):
        ret = self.r.hget(project_id + "_fusion", fusion_id)
        if not ret:
            print >> sys.stderr, 'no res, return 0'
            return 0
        return ret 
    
    def get_all_fusion_status(self, project_id):
        return self.r.hgetall(project_id + "_fusion")
    
    def rem_fusion_status(self, project_id):
        return self.r.delete(project_id + "_fusion")

    def push_status(self, project_id, task_id, _type, status):
        data = '%s-%s-%s-%s' % (project_id, task_id, _type, status)
        self.r.lpush('stat_queue', data)

    def pop_status(self):
        data = self.r.rpop('stat_queue')
        if data: return data.split('-')
        else: return None



def main():
    q = RedisUtils()
    print q.set_auto_status('5', '26_20180623062901409', 1)
    print q.get_auto_status('5', '26_20180623062901409')
    #print q.set_auto_status('5', '26_20180623062901409', 0)
    print q.get_fusion_status('5', '26_201806230629014094') 
    exit(4)
    exit(2)
    print q.set_state("3", "running")
    data = json.dumps({"id":"auto_111","track_id":"track_111","fid":["xxx","yyy"],"status":"processing"})
    print q.get_auto_status("123", "123_99999_12345")
    print q.set_auto_status("123", "123_99999_54321", data)
    all_info =  q.get_all_auto_status("123")
    print type(all_info)
    print all_info
    exit(1)
    fusion_data = '{"id":"fusion_111","fid":"xxxxx","track_ids":["track_111","track_222"],"status":"cpp binary failed"}'
    print q.set_fusion_status("999", "999_00000_12345", fusion_data)
    print q.get_fusion_status("999", "999_00000_12345")

if __name__ == '__main__':

    q = RedisUtils()
    print q.push_status('p1', '1', 'auto', '0')
    print q.push_status('p1', '2', 'fusion','0')
    print q.push_status('p1', '3', 'position', '0')
    print q.push_status('p1', '4', 'traffic' ,'0')
    print q.pop_status()
    print q.pop_status()
    print q.pop_status()
    print q.pop_status()
    print q.pop_status()
    print q.pop_status()
    print q.pop_status()
    print q.pop_status()



from optparse import OptionParser 
parser = OptionParser() 

add_option = parser.add_option
options = None
args = None

def parse():
	global options
	global args
	if None == options:
		(options, args) = parser.parse_args() 


def option():
	global options
	parse()
	return options

"""
if options.pdcl==True: 
    print 'pdcl is true' 
if options.zdcl==True: 
    print 'zdcl is true' 
"""



#!/usr/bin/env python
#-*-coding: utf-8 -*-

import os
import sys
import json
import gzip
import urllib
import urllib2

#import base_conf
import hdfs_util
from singleton import *
from log import logger
import service_config

host_ip = service_config.GET_CONF('hbase','ngnix_ip')

def unzip(gz_file_path, unzip_file_name):
    try:
        logger().info('unzip gz_file_path[%s] to unzip_file_name[%s]' % (
            gz_file_path, unzip_file_name))
        gz_file = gzip.open(gz_file_path, 'rb')
        unzip_file = open(unzip_file_name, 'w')
        unzip_file.write(gz_file.read())
        os.remove(gz_file_path)
        gz_file.close()
        unzip_file.close()
        return True
    except Exception as e:
        err_msg = traceback.format_exc()
        logger().error(err_msg)
        return False

@singleton
class TrackManager():
    def __init__(self):
        self.wirte_batch_url = 'http://%s/track/batch/set'%(host_ip)
        self.get_newest_batch_url = 'http://%s/track/batch/get'%(host_ip)
        self.get_track_path_url = 'http://%s/track/location/get'%(host_ip)
        self.save_track_path_url = 'http://%s/track/location/set'%(host_ip)
        self.save_full_track_path_url = 'http://%s/full_reco/track/location/set'%(host_ip)
        self.get_full_track_path_url = 'http://%s/full_reco/track/location/get'%(host_ip)
        self.element_save_track_path_url = 'http://%s/kv/set/v2' %(host_ip)
        self.element_get_track_path_url = 'http://%s/kv/get/v2' %(host_ip)
        self.element_write_batch_url = 'http://%s/kv/set/v2' %(host_ip)
        self.element_get_newest_batch_url = 'http://%s/kv/get/v2' %(host_ip)

    def wirte_batch_to_hbase(self, trackid, batch):
        formdata = {
            "trackId": trackid,
            "batch": batch
        }
        data = urllib.urlencode(formdata)
        req = urllib2.Request(self.wirte_batch_url, data = data)
        ret = urllib2.urlopen(req)
        result = False
        status = ret.getcode()
        if status == 200:
            res = ret.read()
            logger().info('response is %s ,trackid:%s'%(res, trackid))
            result = json.loads(res)['result']
        else:
            logger().error('write batch to hbase fail trackid:%s, batch:%s' \
                            % (trackid, batch))
            return False
        return result

    def wirte_full_batch_to_hbase(self, trackid, batch):
        formdata = {
            "trackId": trackid + '_full',
            "batch": batch
        }
        data = urllib.urlencode(formdata)
        req = urllib2.Request(self.wirte_batch_url, data = data)
        ret = urllib2.urlopen(req)
        result = False
        status = ret.getcode()
        if status == 200:
            res = ret.read()
            logger().info('write full batch response is %s ,trackid:%s'%(res, trackid))
            result = json.loads(res)['result']
        else:
            logger().error('write full batch to hbase fail trackid:%s, batch:%s' \
                            % (trackid, batch))
            return False
        return result

    def get_newest_batch(self, trackid):
        get_batch_url = "%s?trackId=%s"%(self.get_newest_batch_url, trackid)
        
        ret = urllib2.urlopen(get_batch_url)
        batch = None
        status = ret.getcode()
        if status == 200:
            res = ret.read()
            batch = json.loads(res)['result']
        else:
            logger().error('get newest batch request fail status is %s'%status)
            return None
        return batch

    def get_full_track_newest_batch(self, trackid):
        get_batch_url = "%s?trackId=%s"%(self.get_newest_batch_url, trackid + '_full')
        print get_batch_url
        ret = urllib2.urlopen(get_batch_url)
        batch = None
        status = ret.getcode()
        if status == 200:
            res = ret.read()
            batch = json.loads(res)['result']
        else:
            logger().error('get full track newest batch request fail status is %s'%status)
            return None
        return batch

    def get_track_path(self, trackid, batch):
        url = '%s?trackId=%s:%s'%(self.get_track_path_url, trackid, batch)
        ret = urllib2.urlopen(url)
        track_path = None
        status = ret.getcode()
        if status == 200:
            res = ret.read()
            track_path = json.loads(res)['result']
        else:
            logger().error('get track path fail status:%s, trackid:%s, batch:%s'%(status, trackid, batch))
        return track_path

    def get_full_track_path(self, trackid, batch):
        url = '%s?track_id_batch=%s_full:%s'%(self.get_full_track_path_url, trackid, batch)
        print url
        ret = urllib2.urlopen(url)
        track_path = None
        status = ret.getcode()
        if status == 200:
            res = ret.read()
            track_path = json.loads(res)['result']
        else:
            logger().error('get track path fail status:%s, trackid:%s, batch:%s'%(status, trackid, batch))
        return track_path

    #download track return download result and local file path
    def download_track(self, trackid, local_path):
        batch = self.get_newest_batch(trackid)
        if not batch:
            logger().error('batch is None trackid:%s'%(trackid))
            return [False,None]
        track_path = self.get_track_path(trackid, batch)
        if not track_path:
            logger().error('track_path is None trackid:%s, batch:%s'%(trackid, batch))
            return [False,None]
        file_name = track_path.split('/')[-1]
        file_path = '%s/%s'%(local_path, file_name)
        res = hdfs_util.download(track_path, local_path)
        return [res, file_path]

    def download_track_v2(self, trackid, batch, local_path):
        if not batch:
            logger().error('batch is None trackid:%s'%(trackid))
            return [False,None]
        track_path = self.get_track_path(trackid, batch)
        if not track_path:
            logger().error('track_path is None trackid:%s, batch:%s'%(trackid, batch))
            return [False,None]
        file_name = track_path.split('/')[-1] + '_test'
        print file_name
        file_path = '%s/%s'%(local_path, file_name)
        res = hdfs_util.download_for_auto(track_path, local_path)
        return [res, file_path]

    def download_full_track(self, trackid, local_path):
        batch = self.get_full_track_newest_batch(trackid)
        print batch
        if not batch:
            logger().error('batch is None trackid:%s'%(trackid))
            return [False,None]
        track_path = self.get_full_track_path(trackid, batch)
        logger().info('trackid [%s] batch [%s] hdfs path [%s]', trackid, batch, track_path)
        if not track_path:
            logger().error('track_path is None trackid:%s, batch:%s'%(trackid, batch))
            return [False,None]
        file_name = track_path.split('/')[-1]
        file_path = '%s/%s'%(local_path, file_name)
        res = hdfs_util.download(track_path, local_path)
        return [res, file_path]

    def get_full_track_newest_batch(self, trackid):
        #get_batch_url = "%s?trackId=%s"%(self._url_get, trackid + '_full')
        get_batch_url = "%s?trackId=%s"%(self.get_newest_batch_url, trackid + '_full')
        #print get_batch_url
        ret = urllib2.urlopen(get_batch_url)
        batch = None
        status = ret.getcode()
        if status == 200:
            res = ret.read()
            batch = json.loads(res)['result']
        else:
            logger().error('get full track newest batch request fail status is %s'%status)
            return None
        return batch

    def save_track_path(self, row_key, track_path):
        formdata = {"trackId": row_key, "location": track_path}
        data = urllib.urlencode(formdata)
        req = urllib2.Request(self.save_track_path_url, data = data)
        ret = urllib2.urlopen(req)
        status = ret.getcode()
        if status == 200:
            result = ret.read()
            if json.loads(result)['code'] == '0':
                return True
        return False

    def save_full_track_path(self, row_key, track_path):
        formdata = {"track_id_batch": row_key, "track_file_path": track_path}
        data = urllib.urlencode(formdata)
        req = urllib2.Request(self.save_full_track_path_url, data = data)
        ret = urllib2.urlopen(req)
        status = ret.getcode()
        if status == 200:
            result = ret.read()
            if json.loads(result)['code'] == '0':
                return True
        return False

    def element_write_batch_to_hbase(self, trackid, batch):
        data = batch
        write_batch_url = '%s?key=%s&namespace=%s' %(self.element_write_batch_url,
                        trackid, 'element_classification')
        req = urllib2.Request(write_batch_url, data = data)
        ret = urllib2.urlopen(req)
        status = ret.getcode()
        result = False
        if status == 200:
            res = ret.read()
            logger().info('response is %s, trackid:%s' %(res, trackid))
            result = json.loads(res)['result']
        else:
            logger().error('write batch to hbase fail trackid:%s ,batch:%S'
                            % (trackid, batch))
            return False
        return result

    def element_get_newest_batch(self, trackid):
        get_batch_url = '%s?key=%s&namespace=%s' %(self.element_get_newest_batch_url,
                        trackid,'element_classification')
        ret = urllib2.urlopen(get_batch_url)
        batch = None
        status = ret.getcode()
        if status == 200:
            res = ret.read()
            content_type = ret.headers.getheader('Content-Type')
            if 'application/json' == content_type:
                return None
            batch = res.strip()
        else:
            logger().error('get newest batch request fail status is %s' %status)
            return None
        return batch

    def element_save_track_path(self, rowkey, track_path):
        save_track_path_url = '%s?key=%s&namespace=%s'%(self.element_save_track_path_url
                                                        ,rowkey
                                                        ,'element_classification')
        data = track_path
        req = urllib2.Request(save_track_path_url, data = data)
        ret = urllib2.urlopen(req)
        status = ret.getcode()
        if status == 200:
            result = ret.read()
            if json.loads(result)['code'] == '0':
                return True
        return False

    def element_get_track_path(self, rowkey):
        url = '%s?key=%s&namespace=%s' %(self.element_get_track_path_url
                                        ,rowkey ,'element_classification')
        print url
        ret = urllib2.urlopen(url)
        track_path = None
        status = ret.getcode()
        if status == 200:
            res = ret.read()
            content_type = ret.headers.getheader('Content-Type')
            if 'application/json' == content_type:
                return None
            track_path = res.strip()
        else:
            logger().error('get track path fail status:%s, '
                            'trackid:%s' %(status, trackid))
        return track_path

    def element_get_track_data(self, track_id ,batch ,target_path):
        rowkey = '%s-%s' %(track_id, batch)
        track_manager = TrackManager()
        track_path = track_manager.element_get_track_path(rowkey)
        if not track_path:
            logger().error('element path is null, rowkey[%s]' % rowkey)
            return [False, None]
        file_name = track_path.split('/')[-1]
        if track_path.endswith('.gz'):
            file_name = file_name.split('.gz')[0]
        local_file_path = target_path + os.sep + file_name + '_element'
        track_path
        logger().info('track_file_path:%s' %(track_path))
        if track_path.endswith('.gz'):
            gzip_file_name = track_path.split('/')[-1]
            gzip_file_path = target_path + os.sep + gzip_file_name
            res = hdfs_util.download(track_path, target_path)
            if res:
                res = unzip(gzip_file_path, local_file_path)
        else:
            res = hdfs_util.download_sign(track_path, target_path, 'element')
        if not res:
            logger().error('download %s failure' %(track_path))
            return [False, None]
        else:
            logger().info('download %s success' %(track_path))
            return [True, local_file_path]

    def element_get_full_track_data(self, track_id ,target_path):
        full_newest_batch = get_full_track_newest_batch(track_id)
        if not full_newest_batch:
            logger().info('track_id[%s] no full newest batch' % track_id)
            return [Fasle, None]
        batch = '!!!:%s' % full_newest_batch
        rowkey = '%s-%s' %(track_id, batch)
        track_manager = TrackManager()
        track_path = track_manager.element_get_track_path(rowkey)
        if not track_path:
            logger().error('element full pb path is null, rowkey[%s]' % rowkey)
            return [False, None]
        file_name = track_path.split('/')[-1]
        if track_path.endswith('.gz'):
            file_name = file_name.split('.gz')[0]
        local_file_path = target_path + os.sep + file_name + '_full_element'
        logger().info('track_file_path:%s' %(track_path))
        if track_path.endswith('.gz'):
            gzip_file_name = track_path.split('/')[-1]
            gzip_file_path = target_path + os.sep + gzip_file_name
            res = hdfs_util.download(track_path, target_path)
            if res:
                res = unzip(gzip_file_path, local_file_path)
        else:
            res = hdfs_util.download_sign(track_path, target_path, 'full_element')
        if not res:
            logger().error('download %s failure' %(track_path))
            return [False, None]
        else:
            logger().info('download %s success' %(track_path))
            return [True, local_file_path]

def get_track_data(trackid, local_path):
    track_manager = TrackManager()
    return track_manager.download_track(trackid, local_path)

def get_track_data_v2(trackid, batch, local_path):
    track_manager = TrackManager()
    return track_manager.download_track_v2(trackid, batch, local_path)

def write_newest_batch(track_id, batch):
    track_manager = TrackManager()
    return track_manager.wirte_batch_to_hbase(track_id, batch)

def get_newest_batch(track_id):
    track_manager = TrackManager()
    return track_manager.get_newest_batch(track_id)

def get_full_track_data(trackid, local_path):
    track_manager = TrackManager()
    return track_manager.download_full_track(trackid, local_path)

def wirte_full_batch(track_id, batch):
    track_manager = TrackManager()
    return track_manager.wirte_full_batch_to_hbase(track_id, batch)

def get_full_track_newest_batch(track_id):
    track_manager = TrackManager()
    return track_manager.get_full_track_newest_batch(track_id)

def save_track_path(row_key, track_path):
    track_manager = TrackManager()
    return track_manager.save_track_path(row_key, track_path)

def save_full_track_path(row_key, track_path):
    track_manager = TrackManager()
    return track_manager.save_full_track_path(row_key, track_path)

def element_save_track_path(rowkey, track_path):
    track_manager = TrackManager()
    return track_manager.element_save_track_path(rowkey, track_path)

def element_get_track_path(rowkey):
    track_manager = TrackManager()
    return track_manager.element_get_track_path(rowkey)

def element_get_track_data(track_id ,batch ,target_path):
    track_manager = TrackManager()
    return track_manager.element_get_track_data(track_id, batch, target_path)

def element_get_full_track_data(track_id ,target_path):
    track_manager = TrackManager()
    return track_manager.element_get_full_track_data(track_id, target_path)

def element_write_newest_batch(track_id, batch):
    track_manager = TrackManager()
    return track_manager.element_write_batch_to_hbase(track_id, batch)

def element_get_newest_batch(track_id):                                                                                         
    track_manager = TrackManager()
    return track_manager.element_get_newest_batch(track_id)

def main():
    pass

if __name__=='__main__':
    #import base_conf
    track_manager = TrackManager()
    print element_get_track_data('1003263_20180623082322963', '201406393_1', './')
    #print get_track_data_v2('1027016_20180913104303011', '1057330_1', './')
    #print track_manager.download_full_track('26_20180623055908071', '.')
    #print element_get_track_data('1003263_20180623082322963', '201406393_1', './')
    #print element_get_full_track_data('1003259_20180621153951549' ,'.')
    #exit(4)
    #print track_manager.element_get_track_path('1000022_20180730151053997-!!!:201607709_1')
    #print track_manager.element_get_newest_batch('26_20180623081235990')
    #print element_get_full_track_data('26_20180623081235990' , '.')
    #print element_get_track_data('26_20180623081235990', '400059635_1' , './')
    #print element_get_track_data('400047388_2019013012055203', '400052426_1','./')
    #print element_get_track_data('400047388_20190130120552039', '400052426_1','./')
    #main()
    #print track_manager.wirte_batch_to_hbase('99999_20180701073630032','99999_3')
    #print write_newest_batch('99999_20180701073630032','99999_1')
    #print batch_to_hbase('9999', '9999_1')
    #print track_manager.get_newest_batch('99999_20180701073630032')
    #print get_track_data('1017985_20180806104812175','./')
    #print track_manager.get_full_track_newest_batch('9999_128827264')
    #print get_full_track_newest_batch('400051009_20190131123235077')
    #print track_manager.get_full_track_path('400051009_20190131123235077', '400051089_1')
    #print get_full_track_newest_batch('1003259_20180621153951549')
    #print wirte_full_batch('9999_128827264', '999_1')
    #print 'success!'
    #print host_ip
    #print element_get_track_data('1104821_20180624061215589', '333335_1', './')
    #print element_get_track_path('_SUCCESS-333335_1')
    #print element_save_track_path('201377986_20190222105506469-9999_1' ,'/user/hwl/test/201377986_20190222105506469.gz')



# -*- coding:utf-8 -*

import traceback
import time
import redis
from redis.sentinel import Sentinel

from log import logger
from singleton import singleton

hosts = '10.11.5.112:26380, 10.11.5.113:26380, 10.11.5.114:26380, 10.11.5.115:26380, 10.11.5.116:26380'
redis_name = 'redis_master'
redis_password = 'kdgogo'
db_list = 'ir_server,upload'
dblist = {'ir_server_db': 50, 'upload_db': 0}
@singleton
class Pool(object):
    def __init__(self):
        self.__get_sentinel()
        self.__create_redis()

    def __get_sentinel(self):
        sentinel_info = hosts
        sentinels = sentinel_info.split(',')
        sentinel_hosts = []
        for sentinel in sentinels:
            sentinel = sentinel.strip()
            host = sentinel.split(':')[0]
            port = int(sentinel.split(':')[1])
            sentinel_hosts.append((host, port))
        self.sentinel = Sentinel(sentinel_hosts, socket_timeout=15)

    def __create_redis(self):
        redis_name = redis_name
        redis_password = redis_password
        self.__redis_map = {}
        for db_name in db_list.split(','):
            db_name = db_name.strip()
            if not db_name:
                continue
            db_conf_name = db_name + '_db'
            db = dblist[db_conf_name]
            self.__redis_map[db_name] = RobustRedis(self.sentinel, redis_name, db=db,
                                                    redis_password=redis_password)

    def get(self, name):
        assert name in self.__redis_map, 'no redis for[%s]' % name
        return self.__redis_map[name] 

def make_robust(fun, obj, fun_name, recover):
    def new_fun(*args, **kw):
        arg = args
        f = fun
        for i in range(3):
            try:
                return f(*args, **kw)
            except Exception, e:
                logger().warning(traceback.format_exc())
                time.sleep(3)
                recover()
                f = getattr(obj, '__' + fun_name)
                continue
        raise Exception('Redis failed 3 times')
    return new_fun

class RobustRedis(object):
    def __init__(self, sentinel, redis_name, db, redis_password):
        self.__sentinel = sentinel
        self.__redis_name = redis_name
        self.__db = db
        self.__redis_password = redis_password
        self.migrate_robust_fun()

    def __get_master_from_sentinel(self):
        redis_name = self.__redis_name
        db = self.__db
        redis_password = self.__redis_password
        return self.__sentinel.master_for(redis_name, db=db,
                                          password=redis_password,
                                          socket_timeout=10)

    def migrate_robust_fun(self):
        master = self.__get_master_from_sentinel()
        for fun_name in dir(master):
            attr = getattr(master, fun_name)
            if fun_name == 'pipeline':
                setattr(self, fun_name, self.pipeline)
            else:
                if not fun_name.startswith('_'):
                    setattr(self, fun_name,
                            make_robust(attr, self, fun_name, self.recover))
                    setattr(self, '__' + fun_name, attr)

    def pipeline(self, *args, **kw):
        assert False, 'Not implemented'

    def recover(self):
        master = self.__get_master_from_sentinel()
        logger().info('switch master to [%s]', master)
        #self.migrate_robust_fun()

def get(name):
    pool = Pool()
    return pool.get(name)

def redis_pool_main():
    pool = Pool()
    r = pool.get('hbase_index_test')
    for i in range(1000):
        str_i = str(i)
        val = 'a+++' + str_i
        print r.set(str_i, val)

if __name__ == '__main__':
    #redis_pool_main()
    r = Pool().get('ir_server')
    r.set('test', '3')
    print r.get('test')



'''
Created on 20190328
@auther: dq
'''
from setuptools import setup,find_packages
setup(name = 'kd-infra',
      version= '0.1',
      keywords = ('pip', 'infra', 'featureextraction'),
      description = 'Custom script',
      license = 'MIT Licence',
      url = 'https://github.com/duquan640/pip_kd/kd_infra',
      author = 'dq',
      author_email = 'duquan@kuandeng.com',
      packages = find_packages(),
      include_package_data = True,
      platforms = "any"
      #packages = ['kd_infra'],
     )



# -*- coding:utf-8 -*-
import os
import sys
import json
import urllib
import urllib2
import service_config
import requests
import time
#from dump_status import StatusDumper 
from redis_utils import RedisUtils
from log import logger
import StringIO
import gzip


def send_alert(message,title):
    quiet = service_config.GET_CONF('service','alert_pause')
    if quiet == 'True':
        print sys.stderr, 'alert pause, will not send alert'
        return False
    alert_url = service_config.GET_CONF('service','alert_url')
    #alert_url = 'http://op-01.gzproduction.com:9527/api/msg/send'
    title = title
    alert_type = 2
    formdata = {
	"title": title,
	"message": message,
	"alert_type": alert_type,
        "to_party": 20
    }
    data = urllib.urlencode(formdata)
    req = urllib2.Request(alert_url, data = data)
    ret = urllib2.urlopen(req)
    res = ret.read()
    response = json.loads(res)
    return '0' == response['code']

def set_status_to_hbase(project_id, taskid, status, returncode, _type):
    for i in range(3):
        ret = __set_status_to_hbase(project_id, taskid, status, returncode, _type)
        logger().info('time [%d] insert status to hbase ret [%s]', i + 1, ret)
        if ret:
            logger().info('insert status to hbase success')
            return True
    return False

def __set_status_to_hbase(project_id, taskid, status, returncode, _type):
    set_status_url = service_config.GET_CONF("service", "set_status_url")
    payload = {}
    key = 'jaguar_status_%s_%s_%s' % (project_id, _type, taskid)
    payload["key"] = key 
    value = '%s#%s' % (status, returncode)
    payload["value"] = value
    #logger().info('post key [%s] value [%s]', key, value)
    res = requests.post(set_status_url, data=payload)
    if res.status_code != 200:
        logger().error("return code is not 200")
        return False
    try:
        jdata = res.json()
        logger().info('key [%s] jdata [%s]', key, str(jdata))
    except Exception, e:
        import traceback
        traceback.print_exc(e)
        return False
    return jdata['code'] == '0'

def get_status_from_hbase(project_id, taskid, _type):
    start_time = time.time()
    get_status_url = service_config.GET_CONF("service", "get_status_url")
    payload = {}
    key = '%s_%s_%s' % (project_id, _type, taskid)
    payload["key"] = key 
    res = requests.get(get_status_url, data=payload)
    if res.status_code != 200:
        logger().error("return code is not 200 vs %s", str(res.status_code))
        return False
    try:
        jdata = res.json()
        if jdata['code'] != '0':
            return False
        end_time = time.time()
        const_time = end_time - start_time
        #logger().info('get info key [%s], value [%s] const time [%s]', 
        #                key, str(jdata['result']), const_time)
        return jdata['result']
    except Exception, e:
        import traceback
        traceback.print_exc(e)

def get_track_info(taskid):
    get_status_url = service_config.GET_CONF("service", "get_track_info_url")
    payload = {}
    payload["track_id"] = taskid
    res = requests.post(get_status_url, data=payload)
    if res.status_code != 200:
        logger().error("return code is not 200 vs %s", str(res.status_code))
        return False
    try:
        jdata = res.json()
    except Exception, e:
        import traceback
        traceback.print_exc(e)
        return False
    return json.dumps(jdata['result'])

def get_all_status(project_id):
    get_status_url = service_config.GET_CONF("service", "get_all_status_url")
    payload = {}
    payload["projectId"] = project_id
    res = requests.post(get_status_url, data=payload)
    if res.status_code != 200:
        logger().error("return code is not 200 vs %s", str(res.status_code))
        return False
    try:
        jdata = res.json()
    except Exception, e:
        import traceback
        traceback.print_exc(e)
        return False
    return json.dumps(jdata['result'])

def get_version():
    #url = 'http://op-01.gzproduction.com:9527/api/service_version/get_all'
    url = service_config.GET_CONF("service", "service_versions_url")
    print url
    res = requests.post(url)
    if res.status_code != 200:
        logger().error("return code is not 200 vs %s", str(res.status_code))
        return None
    try:
        jdata = res.json()
        re = jdata['result']
        results = {}
        if len(re['fusion']) > 0 and len(re['auto']) > 0 and len(re['ground']) > 0 and len(re['pole']) >0:
            results['fusion'] = re['fusion'][0]
            results['auto'] = re['auto'][0]
            results['ground'] = re['ground'][0]
            results['pole'] = re['pole'][0]
        else:
            logger().error("version fusion: %s, auto: %s, ground: %s, pole: %s",  ( re['fusion'],re['auto'], re['ground'], re['pole'] ))
            return None
    except Exception, e:
        import traceback
        traceback.print_exc(e)
        return None
    return results

def build_log_link():
    CONTAINER_ID = os.getenv('CONTAINER_ID')
    NM_HOST = os.getenv('NM_HOST')
    mapred_task_id = os.getenv('mapred_task_id')
    NM_PORT = os.getenv('NM_PORT')
    base_url = ('http://hm-001:19888/jobhistory/logs'
                '/%s:%s/%s/%s/hadoop/stderr/') % (NM_HOST, NM_PORT, CONTAINER_ID,
                        mapred_task_id)
    logger().info('jobhistory url : %s', base_url)
    return base_url

#status = start or stop
def dotting(project_id, taskid, _type, status):
    for i in range(3):
        ret = __dotting(project_id, taskid, _type, status)
        logger().info('time [%d] insert status to hbase ret [%s]', i + 1, ret)
        if ret:
            logger().info('insert status to hbase success')
            return True
        time.sleep(3)
    return False

def __dotting(project_id, taskid, _type, status):
    if status == 'start':
        dotting_url = service_config.GET_CONF('service','dotting_url')
    else:
        dotting_url = service_config.GET_CONF('service','dotting_stop_url')
    #payload = {}
    #payload['pid'] = project_id 
    #payload['stage'] = _type
    #payload['taskid'] = taskid
    try:
        dotting_url = '%s?pid=%s&stage=%s&taskid=%s' % (dotting_url, project_id, _type, taskid)
        logger().info('post pid [%s] stage [%s], taskid [%s], url [%s] status [%s]', 
                    project_id, _type, taskid, dotting_url, status)
        res = requests.post(dotting_url)
    except:
        logger().error("get a exception")
        return False
    if res.status_code != 200:
        logger().error("return code is not 200 vs %s", str(res.status_code))
        logger().error('error info [%s]', str(res.json))
        return False
    try:
        jdata = res.json()
        logger().info('jdata [%s]', str(jdata))
    except Exception, e:
        import traceback
        traceback.print_exc(e)
        return False
    return jdata['code'] == '0'

def dotting_all(project_id, data):
    for i in range(3):
        ret = __dotting_all(project_id, data)
        logger().info('time [%d] insert status to hbase ret [%s]', i + 1, ret)
        if ret:
            logger().info('insert status to hbase success')
            return True
        time.sleep(3)
    return False

def __dotting_all(project_id, data):
    dotting_url = service_config.GET_CONF('service','dotting_all_url')
    taskids = json.dumps(data) 
    dotting_url = '%s?pid=%s' % (dotting_url, project_id)
    logger().info('post pid [%s] taskids [%s], url [%s]', 
                    project_id, str(taskids), dotting_url)
    try:
        res = requests.post(dotting_url, data=taskids)
    except:
        logger().info('__dotting_all get an exception')
        return False
    if res.status_code != 200:
        logger().error("return code is not 200 vs %s", str(res.status_code))
        logger().error('error info [%s]', str(res.json))
        return False
    try:
        jdata = res.json()
        logger().info('jdata [%s]', str(jdata))
    except Exception, e:
        import traceback
        traceback.print_exc(e)
        return False
    return jdata['code'] == '0'
    
def dotting_stage(project_id, stage):
    for i in range(3):
        ret = __dotting_stage(project_id, stage)
        logger().info('time [%d] insert status to hbase ret [%s]', i + 1, ret)
        if ret:
            logger().info('insert status to hbase success')
            return True
        time.sleep(3)
    return False

def __dotting_stage(project_id, stage):
    dotting_url = service_config.GET_CONF('service','dotting_stage_url')
    print dotting_url
    dotting_url = '%s?pid=%s&stage=%s' % (dotting_url, project_id, stage)
    logger().info('post pid [%s] url [%s], stage [%s]', 
                    project_id, dotting_url, stage)
    try:
        res = requests.post(dotting_url)
    except:
        logger().info('__dotting_stage get an exception')
        return False
    if res.status_code != 200:
        logger().error("return code is not 200 vs %s", str(res.status_code))
        logger().error('error info [%s]', str(res.json))
        return False
    try:
        jdata = res.json()
        logger().info('jdata [%s]', str(jdata))
    except Exception, e:
        import traceback
        traceback.print_exc(e)
        return False
    return jdata['code'] == '0'

def gzip_compress(buf):
    out = StringIO.StringIO()
    with gzip.GzipFile(fileobj=out, mode="w") as f:
        f.write(buf)
    return out.getvalue()

def get_cpp_mem(taskid):
    show_cmd = 'ps aux | grep -v grep| grep %s | awk \'{print $6}\'' % taskid
    print show_cmd
    data = os.popen(show_cmd, 'r').read()
    ret = data.strip().split('\n')[1:]
    for line in ret:
        print line

def main():
    #print dotting_stage('300000210', 'auto')
    #exit(4) 
    get_cpp_mem('400115432')
    #print dotting('1', '1', 'auto', 'stop')
    exit(4) 
    print dotting_all('300000210',{})
    exit(4) 
    #print send_alert("6666", "bpl")
    #print set_status_to_hbase('300000138', '300005683', '0', '0', 'traffic')
    #print get_status_from_hbase('300000138', '300005683', 'traffic')
    #print get_track_info('1026990_20180905143247856')
    #print build_log_link()
    #print get_all_status('567')

if __name__ == '__main__':
    main()
    print(get_version())



#!/usr/bin/python
# -*- coding: utf-8 -*-

import time
import os
import requests


if os.getenv('PYTHONPATH') and 'py_path_managers/projects' in os.getenv('PYTHONPATH'):
    import jaguar_v2
    jaguar_v2.sys_append_paths()

from bases.log import logger
from bases.service_config import GET_CONF

def append(file_path, data):
    f = open(file_path, 'a')
    f.write(data)
    f.write('\n')
    f.close()

def write(file_path, data):
    f = open(file_path, 'w')
    f.write(data)
    f.write('\n')
    f.close()

def read(file_path):
    raw_lines = [line.strip() for line in open(file_path)]
    lines = [line for line in raw_lines if line]
    return lines

class Client(object):
    def __init__(self, queue_name, url):
        self.__queue_name = queue_name
        self.__push_url = url + '/citroen/push?queue=%s' % self.__queue_name
        self.__pop_url = url + '/citroen/pop?queue=%s' % self.__queue_name
        self.__len_url = url + '/citroen/length?queue=%s' % self.__queue_name
        self.__popall_url = url + '/citroen/popall?queue=%s' % self.__queue_name
        self.__delete_url = url + '/citroen/delete?queue=%s' % self.__queue_name

    # param data could be a string or a list of string
    def push(self, data, project_id, priority=GET_CONF('priority', 'NORMAL')):
        push_url = self.__push_url
        push_url += '&project_id=%s&priority=%s' % (project_id, priority)
        res = requests.post(push_url, data=data)
        if res.status_code != 200:
            logger().error("return code is not 200")
            return False
        jdata = res.json()
        if jdata['code'] != '0':
            logger().error("json code is[%s]", str(jdata['code']))
            return False
        logger().info('push succ. queue: %s, proj: %s, priority: %s, data: %s',
                self.__queue_name, project_id, priority, data[:100])
        return True

    def pop(self, priority):
        pop_url = self.__pop_url
        pop_url += '&priority=%s' % (priority)
        res = requests.get(pop_url)
        if res.status_code != 200:
            logger().error("return code is not 200")
            return None
        jdata = res.json()
        if jdata['code'] != '0':
            logger().error("json code is[%s]", str(jdata['code']))
            return None

        logger().info('pop succ. queue: %s, priority: %s',
                self.__queue_name, priority)
        return jdata['result']

    def length(self, priority=''):
        len_url = self.__len_url
        len_url += '&priority=%s' % (priority)
        res = requests.get(len_url)
        if res.status_code != 200:
            raise Exception("return code is not 200")
        jdata = res.json()
        if jdata['code'] != '0':
            raise Exception("json code is[%s]", str(jdata['code']))

        logger().info('length succ. queue: %s, priority: %s, resp: %s',
                self.__queue_name, priority, jdata['result'])
        return jdata['result']

    def pop_all(self, priority=''):
        popall_url = self.__popall_url
        popall_url += '&priority=%s' % (priority)
        res = requests.get(popall_url)
        if res.status_code != 200:
            raise Exception("return code is not 200")
            return None
        jdata = res.json()
        if jdata['code'] != '0':
            raise Exception("json code is [%s]", str(jdata['code']))

        logger().info('pop all succ. queue: %s, priority: %s, resp: %s',
                self.__queue_name, priority, jdata['result'][:100])
        return jdata['result']

    def delete(self, project_id):
        delete_url = self.__delete_url + '&project_id=' + project_id
        res = requests.post(delete_url)
        if res.status_code != 200:
            logger().error("return code is not 200")
            return False
        jdata = res.json()
        if jdata['code'] != '0':
            logger().error("json code is[%s]", str(jdata['code']))
            return False

        logger().info('delete succ. queue: %s, proj: %s, resp: %s',
                self.__queue_name, project_id, jdata['result'][:100])
        return True


def get_client(name):
    return Client(name, url=GET_CONF('citroen', 'url'))
    
def main():
    VERY_HIGH = GET_CONF('priority', 'VERY_HIGH')
    HIGH = GET_CONF('priority', 'HIGH')
    NORMAL = GET_CONF('priority', 'NORMAL')

    client = get_client('test')
    print 'pop all:', client.pop_all()
    client.push('1', '12345', VERY_HIGH)
    client.push('2', '12345', HIGH)
    client.push('3', '12345', NORMAL)
    print 'pop all very high:', client.pop_all(VERY_HIGH)
    print 'pop all high:', client.pop_all(HIGH)
    print 'pop all normal:', client.pop_all(NORMAL)

    client.push('1', '12345', VERY_HIGH)
    client.push('2', '12345', HIGH)
    client.push('3', '12345', NORMAL)
    print 'len: ', client.length()
    print 'len very high: ', client.length(VERY_HIGH)
    print 'len high: ', client.length(HIGH)
    print 'len normal: ', client.length(NORMAL)
    print 'pop very high:', client.pop(VERY_HIGH)
    print 'pop high:', client.pop(HIGH)
    print 'pop normal:', client.pop(NORMAL)
    print 'len: ', client.length()
    exit(0)

    client.push('4', '23456')
    client.push('5', '23456')
    client.delete('23456')

    client.push('6', '45678')
    client.push('7', '45678')
    print 'pop:', client.pop()
    client.push('8', '56789')
    client.push('9', '00000')
    client.delete('45678')

    print 'pop all:', client.pop_all()
    print 'pop all:', client.pop_all()

 
if __name__ == '__main__':
    main()
